<?php
// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class FlowMattic_Email_Parser {

	/**
	 * Constructor.
	 *
	 * @access public
	 * @since 2.2.0
	 * @return void
	 */
	public function __construct() {
		// Enqueue custom view for maths.
		add_action( 'flowmattic_enqueue_views', array( $this, 'enqueue_views' ) );

		flowmattic_add_application(
			'email_parser',
			array(
				'name'         => esc_attr__( 'Email Parser by FlowMattic', 'flowmattic' ),
				'icon'         => FLOWMATTIC_PLUGIN_URL . 'inc/apps/email-parser/icon.svg',
				'instructions' => 'Send your emails to the above email. FlowMattic will parse your email and return the data. Attachments upto 5MB are supported.',
				'actions'      => array(),
				'triggers'     => $this->get_triggers(),
				'base'         => 'core',
				'type'         => 'trigger',
			)
		);
	}

	/**
	 * Enqueue view js.
	 *
	 * @access public
	 * @since 2.2.0
	 * @return void
	 */
	public function enqueue_views() {
		wp_enqueue_script( 'flowmattic-app-view-email-parser', FLOWMATTIC_PLUGIN_URL . 'inc/apps/email-parser/view-email-parser.js', array( 'flowmattic-workflow-utils' ), FLOWMATTIC_VERSION, true );
	}

	/**
	 * Set actions.
	 *
	 * @access public
	 * @since 2.2.0
	 * @return array
	 */
	public function get_triggers() {
		return array(
			'email_received' => array(
				'title'       => esc_attr__( 'Email Received', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when an email is received on the email parser email.', 'flowmattic' ),
			),
		);
	}

	/**
	 * Set actions.
	 *
	 * @access public
	 * @since 2.2.0
	 * @param string $workflow_id Workflow ID.
	 * @return string
	 */
	public static function get_mailhook_url( $workflow_id ) {
		$domain         = wp_parse_url( home_url() );
		$encoded_domain = base64_encode( $domain['host'] ); // @codingStandardsIgnoreLine
		$encoded_domain = str_replace( '=', '', $encoded_domain );
		$mailhook_url   = $workflow_id . $encoded_domain . '@flowmatticmail.com';

		return $mailhook_url;
	}
}

new FlowMattic_Email_Parser();
