/* global FlowMatticWorkflow, FlowMatticWorkflowEvents, FlowMatticWorkflowApp, FlowMatticWorkflowSteps */
var FlowMatticWorkflow = FlowMatticWorkflow || {};

( function( $ ) {

	jQuery( document ).ready( function() {
		// Workflow Trigger WordPress View.
		FlowMatticWorkflow.WordpressView = Backbone.View.extend( {
			template: FlowMatticWorkflow.template( jQuery( '#flowmattic-application-wordpress-data-template' ).html() ),

			events: {
				'click .wordpress-data-toggle': 'toggleResponseData',
				'change [name*="user-field-"]': 'updateFieldValues',
				'click .wp-generate-pw': 'generatePassword'
			},

			initialize: function() {
				// Listen to form submission.
				this.listenTo( FlowMatticWorkflowEvents, 'formSubmissionReceived', this.updateFormSubmissionData );
			},

			render: function() {
				var thisEl = this,
					applicationSettings = {},
					submissionData = {},
					appActionTemplate = '',
					appAction = this.model.get( 'action' );

				if ( 'action' === this.model.get( 'type' ) && '' !== appAction ) {
					appActionTemplate = FlowMatticWorkflow.template( jQuery( '#flowmattic-wordpress-' + appAction + '-action-template' ).html() );
					this.$el.html( appActionTemplate( this.model.toJSON() ) );
				} else {
					this.$el.html( this.template( this.model.toJSON() ) );

					if ( 'page_view' === appAction ) {
						appActionTemplate = FlowMatticWorkflow.template( jQuery( '#flowmattic-wordpress-' + appAction + '-trigger-template' ).html() );
						jQuery( this.$el ).find( '.flowmattic-wordpress-trigger-data' ).html( appActionTemplate( this.model.toJSON() ) );
					}

					if ( 'undefined' !== typeof this.model.get( 'applicationSettings' ) ) {
						applicationSettings        = this.model.get( 'applicationSettings' ).webhook_capture;
						submissionData.captureData = applicationSettings;
						this.updateFormSubmissionData( submissionData );
					}
				}

				if ( 'undefined' !== typeof this.model.get( 'capturedData' ) ) {
					capturedData = this.model.get( 'capturedData' );
					submissionData.capturedData = capturedData;
					submissionData.stepID = this.model.get( 'stepID' );

					FlowMatticWorkflowEvents.trigger( 'eventResponseReceived', submissionData, submissionData.stepID );
				}

				this.$el.find( 'select' ).selectpicker();

				this.setFormOptions();

				setTimeout( function() {
					if ( thisEl.$el.find( '.flowmattic-content-editor' ).length ) {
						var editorTemplate = FlowMatticWorkflow.template( jQuery( '#flowmattic-text-editor-template' ).html() ),
							editorWrapper = thisEl.$el.find( '.flowmattic-content-editor' );

						// Initialize the editor.
						editorWrapper.html( editorTemplate( thisEl.model.toJSON() ) );

						editorWrapper.find( '[data-toggle="tooltip"]' ).tooltip( { template: '<div class="tooltip" role="tooltip"><div class="tooltip-inner"></div></div>' } );

						// Handle the editor commands.
						window.actionView.handleEditorCommands( thisEl );
					}
				}, 500 );

				return this;
			},

			updateFieldValues: function( event ) {
				var wpField = jQuery( event.target ),
					name = wpField.attr( 'data-field' ),
					value = wpField.val();

				if ( wpField.is( ':checkbox' ) ) {
					value = wpField.is( ':checked' ) ? 'Yes' : 'No';
				}

				if ( 'auto_password_generation' === name ) {
					jQuery( this.$el ).find( '.wp-custom-password' ).hide();

					if ( 'Yes' !== value ) {
						jQuery( this.$el ).find( '.wp-custom-password' ).show();
					}
				}

				if ( 'post_type' === name ) {
					jQuery( this.$el ).find( '.fm-post-taxonomies' ).hide();

					if ( 'post' === value ) {
						jQuery( this.$el ).find( '.fm-post-taxonomies' ).show();
					}
				}

				if ( 'object' === typeof value && -1 !== jQuery.inArray( 'fm-reset', value ) ) {
					wpField.selectpicker( 'val', '' );
					wpField.selectpicker( 'refresh' );
					wpField.selectpicker( 'hide' );
				} else if ( 'fm-reset' === value ) {
					wpField.selectpicker( 'val', '' );
					wpField.selectpicker( 'refresh' );
				}

				this.model.set( name, value );

				FlowMatticWorkflowEvents.trigger( 'actionAppDataUpdateSingleAttribute', name, value, this );

				FlowMatticWorkflowEvents.trigger( 'saveWorkflowDraft' );
			},

			toggleResponseData: function() {
				var toggleLink = jQuery( this.$el ).find( '.wordpress-data-toggle' ),
					toggleDataWrap = toggleLink.next( '.wordpress-response-body' );

				toggleLink.toggleClass( 'toggle' );
				toggleDataWrap.toggle( 'slideTop' );
			},

			updateFormSubmissionData: function( response ) {
				var responseTemplate = FlowMatticWorkflow.template( jQuery( '#flowmattic-wordpress-response-template' ).html() );

				this.$el.find( '.fm-wordpress-capture-data' ).html( responseTemplate( response ) );

				FlowMatticWorkflowEvents.trigger( 'saveWorkflowDraft' );
			},

			setFormOptions: function() {
				var elements = jQuery( this.$el ).find( '.flowmattic-wordpress-form-data' ),
					currentFormAction = this.model.get( 'action' );

				elements.hide();

				if ( 'post' === this.model.get( 'post_type' ) ) {
					jQuery( this.$el ).find( '.fm-post-taxonomies' ).show();
				} else {
					jQuery( this.$el ).find( '.fm-post-taxonomies' ).hide();
				}

				if ( '' !== currentFormAction ) {
					jQuery( this.$el ).find( '.flowmattic-wordpress-form-data' ).show();
				}
			},

			triggerParentDataUpdate: function() {
				var formSubmission = this.model.get( 'capturedData' );

				setTimeout( function() {
					FlowMatticWorkflowEvents.trigger( 'triggerAppDataUpdated', formSubmission );
				}, 200 );
			},

			generatePassword: function() {
				var thisEl = this;

				Backbone.ajax( {
					url: ajaxurl,
					data: { action: 'generate-password' },
					type: 'POST',
					success: function( response ) {
						jQuery( thisEl.$el ).find(  '.wordpress-password' ).val( response.data ).trigger( 'change' );
					}
				} );
			}
		} );
	} );
}( jQuery ) );
